% W-H system simulation program
% nonlinear system y = av1*x + av2*x^2 + av3*x^3 + av4*x^4+ av5*x^5,
clear all; close all; rand('seed',0);
%
NLG=.2;
av=[1 NLG*[0.1 0.2 0.05 0.02]];
Plevel=2; Plevel2=Plevel^2;               % level of input
ONlevel=0.0;                              % output noise level
% excitation signals: 'rms' - multisine

ML=100;                   %number of measurements
npoints=1024*2*4; 		% number of time domain points
n=npoints/2;     		% maximum number of frequencies
nfreq=fix(npoints/10);
junit=sqrt(-1);
%
% frequency axes
fv=[1:nfreq]';
%
% W-H system design, 'L' input dynamics, 'R' output dynamics
[b1,a1]=butter(3,0.03,'high'); 
[b2,a2]=butter(3,0.07);

fvrel=0.1*fv/nfreq; zet=exp(-sqrt(-1)*2*pi*fvrel);
HLf=(b1(1)+b1(2)*zet+b1(3)*zet.^2+b1(4)*zet.^3)./(a1(1)+a1(2)*zet+a1(3)*zet.^2+a1(4)*zet.^3);
HRf=(b2(1)+b2(2)*zet+b2(3)*zet.^2+b2(4)*zet.^3)./(a2(1)+a2(2)*zet+a2(3)*zet.^2+a2(4)*zet.^3);
LRf=HLf.*HRf;
%
% related dynamic systems
RDS1=1; RDS3=3*sum(abs(HLf(fv)).^2)/length(fv);
RDS5=5*3*(sum(abs(HLf(fv)).^2)/length(fv))^2;
RDStr=av(1)*RDS1+Plevel2*av(3)*RDS3+Plevel2^2*av(5)*RDS5;

figure(1),
 subplot(2,2,1), plot(fvrel,db(HLf)); grid; axis([0 .1 -50 10]);
 title('Input System','Fontsize',12);
 subplot(2,2,2), plot(fvrel,db(HRf)); grid; axis([0 .1 -50 10]);
 title('Output System','Fontsize',12);
 subplot(2,2,4), plot(fvrel,db(LRf)); grid; axis([0 .1 -50 10]);
 title('Linear System','Fontsize',12);
%
Xrms=zeros(npoints,1); X1rms=Xrms; X2rms=Xrms; 
Grms=zeros(length(fv),1); GrmsAv=Grms; 
%
for ll=1:ML, ll
% xrms, Xrms - random phase multisine
fi=rand(length(fv),1)*2*pi; Zx=exp(-junit*fi); Xrms=0*Xrms;
Xrms(fv+1)=Zx; Xrms(2*n+1-fv)=conj(Xrms(1+fv));
xrms=real(ifft(Xrms)); Ax=sqrt(sum(xrms.^2)/npoints); xrms=xrms/Ax;
 
xrms=Plevel*xrms; 
Xrms=fft(xrms); 
%pause
% passing signals through input dynamics
X1rms(fv+1)=Xrms(fv+1).*HLf;
X1rms(2*n+1-fv)=conj(X1rms(1+fv));
x1rms=real(ifft(X1rms));

% passing signals through nonlinear system
x2rms=0*x1rms;  
if av(1)~=0, x2rms=x2rms+av(1)*x1rms; end;
if av(2)~=0, x2rms=x2rms+av(2)*x1rms.^2; end;
if av(3)~=0, x2rms=x2rms+av(3)*x1rms.^3; end;
if av(4)~=0, x2rms=x2rms+av(4)*x1rms.^4; end;
if av(5)~=0, x2rms=x2rms+av(5)*x1rms.^5; end;
%
% passing signals through output dynamics
X2rms=fft(x2rms); X2rmsB=X2rms(fv+1);
YrmsBR=X2rmsB.*HRf;

YY(fv+1)=YrmsBR;
YY(2*n+1-fv)=conj(YY(1+fv));
yy=real(ifft(YY));

yy=yy+ONlevel*randn(size(yy));

YY2=fft(yy); YY2=YY2(fv+1).';

% computing estimates of the system transfer characteristics
Grms=YY2./Xrms(fv+1);
GrmsAv=GrmsAv+(Grms-GrmsAv)/ll;

figure(6),
plot(fv,db(RDStr*LRf),'g',fv,db(Grms(:)),'k',fv,db(GrmsAv(:)),'b');
axis([0 1000 -30 10]); grid; title('RPhMS','Fontsize',12); drawnow;

end;

figure(7),
plot(fv,db(LRf),'r',fv,db(RDStr*LRf),'g',fv,db(GrmsAv(:)),'b',fv,db(Grms(:)),'k');
axis([0 1000 -30 10]); grid; 
xlabel('                                  Relativ frekvencia','Fontsize',12);
ylabel('                                  G(f)                 dB','Fontsize',12);
title(['\sigma  = ', num2str(ONlevel), ',    Plevel = ', num2str(Plevel), ',    NL = ', num2str(NLG)],'Fontsize',12);



